
#import "IGXMLParser.h"

@interface IGXMLParser()<NSXMLParserDelegate>
{
    NSMutableDictionary* _xmlContents;
    NSMutableArray* _currentLevel;
    NSString* _lastElementName;
    BOOL _needRemoval;
}

@end

@implementation IGXMLParser

+(NSDictionary*)parse:(NSData *)data
{
    IGXMLParser* xp = [[IGXMLParser alloc]init];
    NSXMLParser* parser = [[NSXMLParser alloc]initWithData:data];
    parser.delegate = xp;
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    [parser parse];
    
    return xp->_xmlContents;
}

-(void)parserDidStartDocument:(NSXMLParser *)parser
{
    _xmlContents = [[NSMutableDictionary alloc]init];
    _currentLevel = [[NSMutableArray alloc]init];
    [_currentLevel addObject:_xmlContents];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    _lastElementName = elementName;
    
    NSString * name;
    NSString * secureStatus;
    NSString * portNumber;
    NSString * value;
    NSString * state;
    NSString * resourceType;
    //For Disk Test
    NSString *idVal;
    NSString *model;
    NSString *serialNum;
    NSString *firmVersion;
    NSString *RPM;
    NSString *sectors;
    NSString *capacity;
    NSString *temperature;
    NSString *ADAError;
    NSString *slotName;
    NSString *hardwareInterface;
    NSString *volName;
    NSString *volState;
    NSString *volHostID;
    NSString *diskState;
    NSString *channel;
    //Volume_Collection Type
    NSString* RAIDLevel;
    
    //Check USB Connected
    NSString* isDeviceConnected;
    NSString *resource_id, * resource_type;
    
    //USB Info
    NSString* size;
    NSString* free;
    NSString* USBName;
    //For ReadyCloud...
    NSString * nameTag;
    NSString * enabledStatus;
    
    //For NAS_Health status...
    NSString * min_fan;
    NSString * max_fan;
    
    //Log Collection...
    NSString *eventId;
    NSString *timestamp;
    NSString *localized_ts;
    NSString *severity;
    NSString *category;
    NSString *message_tag;
    NSString *keyDict;
    NSString *log_entries;
    NSString *currentLogEntries;
    NSString *alertContact;
    NSString *user;
    NSString* server;
    NSString* port;
    NSString* from,*tls;
    NSString* password;
    
    //AlertSettingsConfig
    NSString *volume_ac;
    NSString *disk_failure;
    NSString *fan;
    NSString *power;
    NSString *board_temp;
    NSString *ups;
    NSString *disk_full;
    NSString *disk_temp;
    NSString *quota_exceed;
    NSString *psu;
    
    NSMutableDictionary *args;
    NSString *share_TypeShare;
    
    if ([elementName isEqual:@"xs:error-details"])
    {
        name = [attributeDict objectForKey:@"xs:error-details"];
    }
    
    if([elementName isEqual:@"xs:response"])
    {
        
        // then you just need to grab each of your attributes
        name = [attributeDict objectForKey:@"status"];
        
        
    }
    
    if([elementName isEqual:@"Log_Collection"])
    {
        
        // then you just need to grab each of your attributes
        log_entries = [attributeDict objectForKey:@"n_total_entries"];
        currentLogEntries=[attributeDict objectForKey:@"n_entries"];
        
    }
    
    if([elementName isEqual:@"Share"])
    {
        // then you just need to grab each of your attributes
        share_TypeShare = [attributeDict objectForKey:@"resource-type"];
    }
    
    if([elementName isEqual:@"AlertConfig"])
    {
        
        // then you just need to grab each of your attributes
        alertContact = [attributeDict objectForKey:@"contact"];
        user = [attributeDict objectForKey:@"user"];
        password = [attributeDict objectForKey:@"pass"];
        server = [attributeDict objectForKey:@"server"];
        port = [attributeDict objectForKey:@"port"];
        from = [attributeDict objectForKey:@"from"];
        tls  = [attributeDict objectForKey:@"tls"];
        
        
    }
    if([elementName isEqual:@"Monitor"])
    {
        
        // then you just need to grab each of your attributes
        volume_ac = [attributeDict objectForKey:@"volume"];
        disk_failure = [attributeDict objectForKey:@"disk_failure"];
        fan = [attributeDict objectForKey:@"fan"];
        power = [attributeDict objectForKey:@"power"];
        board_temp = [attributeDict objectForKey:@"board_temp"];
        ups = [attributeDict objectForKey:@"ups"];
        disk_full = [attributeDict objectForKey:@"disk_full"];
        disk_temp = [attributeDict objectForKey:@"disk_temp"];
        quota_exceed = [attributeDict objectForKey:@"quota_exceed"];
        psu = [attributeDict objectForKey:@"psu"];
        
    }
    
    //    if([elementName isEqual:@"xs:get-s"])
    //    {
    //        
    //        // then you just need to grab each of your attributes
    //        resourceType = [attributeDict objectForKey:@"resource-type"];
    //        
    //        
    //    }
    
    if ([elementName isEqual:@"Volume"])
    {
        isDeviceConnected = [attributeDict objectForKey:@"ejectable"];
        
    }
    if ([elementName isEqual:@"volume"])
    {
        free = [attributeDict objectForKey:@"free"];
        size = [attributeDict objectForKey:@"size"];
        NSString* mount = [attributeDict objectForKey:@"mount"];
        NSArray* components = [mount componentsSeparatedByString:@"/"];
        USBName = components[components.count-1];
        
        
    }
    
    if ([elementName isEqual:@"Log"])
    {
        eventId = [attributeDict objectForKey:@"id"];
        timestamp = [attributeDict objectForKey:@"timestamp"];
        localized_ts = [attributeDict objectForKey:@"localized_ts"];
        severity = [attributeDict objectForKey:@"severity"];
        category = [attributeDict objectForKey:@"category"];
        message_tag = [attributeDict objectForKey:@"message-tag"];
    }
    
    if ([elementName isEqual:@"args"])
    {
        // eventId = [attributeDict objectForKey:@"id"];
        
        NSArray *arrKey=[attributeDict allKeys];
        args=[[NSMutableDictionary alloc]init];
        [arrKey enumerateObjectsUsingBlock:^(NSString *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            
            [args setObject:[attributeDict objectForKey:obj] forKey:obj];
            
        }];
        
    }
    
    if ([elementName isEqual:@"Language"])
    {
        keyDict = [attributeDict objectForKey:@"key"];
    }
    
    
    if ([elementName isEqual:@"RAID"])
    {
        RAIDLevel = [attributeDict objectForKey:@"LEVEL"];
        
    }
    
    if([elementName isEqual:@"FirmwareProgress"])
    {
        if ([attributeDict objectForKey:@"value"] == nil || [attributeDict objectForKey:@"value"] == (id)[NSNull null]) {
            
            value=@"";
        } else {
            value=[attributeDict objectForKey:@"value"];
        }
        state=[attributeDict objectForKey:@"state"];
    }
    
    if([elementName isEqual:@"RemoteAccessSSH"])
    {
        secureStatus=[attributeDict objectForKey:@"enable"];
        portNumber=[attributeDict objectForKey:@"port"];
    }
    
    if([elementName isEqual:@"Disk"])
    {
        idVal=[attributeDict objectForKey:@"resource-id"];
        model=[attributeDict objectForKey:@"Model"];
        serialNum=[attributeDict objectForKey:@"Serial"];
        firmVersion=[attributeDict objectForKey:@"FirmwareVersion"];
        RPM=[attributeDict objectForKey:@"RPM"];
        sectors=[attributeDict objectForKey:@"Sectors"];
        capacity=[attributeDict objectForKey:@"Capacity"];
        temperature=[attributeDict objectForKey:@"Temperature"];
        ADAError=[attributeDict objectForKey:@"ATAError"];
        slotName=[attributeDict objectForKey:@"SlotName"];
        hardwareInterface=[attributeDict objectForKey:@"HardwareInterface"];
        volName=[attributeDict objectForKey:@"PoolName"];
        volState=[attributeDict objectForKey:@"PoolState"];
        volHostID=[attributeDict objectForKey:@"PoolHostId"];
        diskState=[attributeDict objectForKey:@"DiskState"];
        channel=[attributeDict objectForKey:@"Channel"];
    }
    
    if([elementName isEqual:@"Protocol"])
    {     //id="readycloud"
        nameTag=[attributeDict objectForKey:@"id"];
        enabledStatus=[attributeDict objectForKey:@"enabled"];
    }
    if([elementName isEqual:@"FirmwareProgress"])
    {
        if ([attributeDict objectForKey:@"value"] == nil || [attributeDict objectForKey:@"value"] == (id)[NSNull null]) {
            
            value=@"";
        } else {
            value=[attributeDict objectForKey:@"value"];
        }
        state=[attributeDict objectForKey:@"state"];
    }
    
    if([elementName isEqual:@"RemoteAccessSSH"])
    {
        secureStatus=[attributeDict objectForKey:@"enable"];
        portNumber=[attributeDict objectForKey:@"port"];
    }
    
    
    
    if([elementName isEqual:@"Fan"])
    {
        min_fan=[attributeDict objectForKey:@"normal-min"];
        max_fan=[attributeDict objectForKey:@"normal-max"];
    }
    
    
    
    id obj = [_currentLevel objectAtIndex:_currentLevel.count-1];
    
    if(![obj isKindOfClass:[NSDictionary class]])
        return;
    
    NSMutableDictionary* currentLevelLookup = (NSMutableDictionary*)obj;
    NSMutableDictionary* newLevelLookup = [[NSMutableDictionary alloc]init];
    
    // Look and see if we've already found an element with the same name at this level.
    NSObject* currentVal = currentLevelLookup[elementName];
    if(currentVal != nil)
    {
        // We found an element at this level with the same name.
        // Lets make an array out of it.
        
        
        // First lets see if we already made an array.
        if([currentVal isKindOfClass:[NSArray class]])
        {
            
            if([elementName isEqual:@"Protocol"])
            {
                [newLevelLookup setValue:nameTag forKey:@"id"];
                [newLevelLookup setValue:enabledStatus forKey:@"enabled"];
                
            }
            
            if([elementName isEqual:@"Disk"])
            {
                
                [newLevelLookup setValue:idVal forKey:@"resource-id"];
                [newLevelLookup setValue:model forKey:@"Model"];
                [newLevelLookup setValue:serialNum forKey:@"Serial"];
                [newLevelLookup setValue:firmVersion forKey:@"FirmwareVersion"];
                [newLevelLookup setValue:RPM forKey:@"RPM"];
                [newLevelLookup setValue:sectors forKey:@"Sectors"];
                [newLevelLookup setValue:capacity forKey:@"Capacity"];
                [newLevelLookup setValue:temperature forKey:@"Temperature"];
                [newLevelLookup setValue:ADAError forKey:@"ATAError"];
                [newLevelLookup setValue:slotName forKey:@"SlotName"];
                [newLevelLookup setValue:hardwareInterface forKey:@"HardwareInterface"];
                [newLevelLookup setValue:volName forKey:@"PoolName"];
                [newLevelLookup setValue:volState forKey:@"PoolState"];
                [newLevelLookup setValue:volHostID forKey:@"PoolHostId"];
                [newLevelLookup setValue:diskState forKey:@"DiskState"];
                [newLevelLookup setValue:channel forKey:@"Channel"];
                
            }
            if([elementName isEqual:@"Volume"])
            {
                [newLevelLookup setValue:isDeviceConnected forKey:@"ejectable"];
                [newLevelLookup setValue:isDeviceConnected forKey:@"ejectable"];
                [newLevelLookup setValue:isDeviceConnected forKey:@"ejectable"];
            }
            
            if([elementName isEqual:@"volume"])
            {
                [newLevelLookup setValue:free forKey:@"free"];
                [newLevelLookup setValue:size forKey:@"size"];
                [newLevelLookup setValue:USBName forKey:@"name"];
            }
            
            //            if([elementName isEqual:@"AlertConfig"])
            //            {
            //                [newLevelLookup setValue:alertContact forKey:@"contact"];
            //            }
            
            if ([elementName isEqual:@"Log"])
            {
                [newLevelLookup setValue:eventId forKey:@"eventId"];
                [newLevelLookup setValue:timestamp forKey:@"timestamp"];
                [newLevelLookup setValue:localized_ts forKey:@"localized_ts"];
                [newLevelLookup setValue:severity forKey:@"severity"];
                [newLevelLookup setValue:category forKey:@"category"];
                [newLevelLookup setValue:message_tag forKey:@"message-tag"];
            }
            if ([elementName isEqual:@"Share"])
            {
                [newLevelLookup setValue:share_TypeShare forKey:@"resource-type"];
            }
            
            if ([elementName isEqual:@"RAID"])
            {
                [newLevelLookup setValue:RAIDLevel forKey:@"Level"];
            }
            
            // We already made an array, so just add the new dictionary we created to the array
            NSMutableArray* vals = (NSMutableArray*)currentVal;
            [vals addObject:newLevelLookup];
            
            
        }
        else
        {
            // Create the array.
            
            if([elementName isEqual:@"Disk"])
            {
                
                [newLevelLookup setValue:idVal forKey:@"resource-id"];
                [newLevelLookup setValue:model forKey:@"Model"];
                [newLevelLookup setValue:serialNum forKey:@"Serial"];
                [newLevelLookup setValue:firmVersion forKey:@"FirmwareVersion"];
                [newLevelLookup setValue:RPM forKey:@"RPM"];
                [newLevelLookup setValue:sectors forKey:@"Sectors"];
                [newLevelLookup setValue:capacity forKey:@"Capacity"];
                [newLevelLookup setValue:temperature forKey:@"Temperature"];
                [newLevelLookup setValue:ADAError forKey:@"ATAError"];
                [newLevelLookup setValue:slotName forKey:@"SlotName"];
                [newLevelLookup setValue:hardwareInterface forKey:@"HardwareInterface"];
                [newLevelLookup setValue:volName forKey:@"PoolName"];
                [newLevelLookup setValue:volState forKey:@"PoolState"];
                [newLevelLookup setValue:volHostID forKey:@"PoolHostId"];
                [newLevelLookup setValue:diskState forKey:@"DiskState"];
                [newLevelLookup setValue:channel forKey:@"Channel"];
                
            }
            if([elementName isEqual:@"Volume"])
            {
                [newLevelLookup setValue:isDeviceConnected forKey:@"ejectable"];
                [newLevelLookup setValue:resource_id forKey:@"Level"];
                [newLevelLookup setValue:resource_type forKey:@"Level"];
                
            }
            
            if([elementName isEqual:@"volume"])
            {
                [newLevelLookup setValue:free forKey:@"free"];
                [newLevelLookup setValue:size forKey:@"size"];
                [newLevelLookup setValue:USBName forKey:@"name"];
            }
            
            if ([elementName isEqual:@"Log"])
            {
                [newLevelLookup setValue:eventId forKey:@"eventId"];
                [newLevelLookup setValue:timestamp forKey:@"timestamp"];
                [newLevelLookup setValue:localized_ts forKey:@"localized_ts"];
                [newLevelLookup setValue:severity forKey:@"severity"];
                [newLevelLookup setValue:category forKey:@"category"];
                [newLevelLookup setValue:message_tag forKey:@"message-tag"];
            }
            if ([elementName isEqual:@"Share"])
            {
                [newLevelLookup setValue:share_TypeShare forKey:@"resource-type"];
            }
            
            if ([elementName isEqual:@"RAID"])
            {
                [newLevelLookup setValue:RAIDLevel forKey:@"Level"];
            }
            
            NSMutableArray* children = [[NSMutableArray alloc]init];
            
            // Add the curren Value first.
            [children addObject: currentVal];
            
            // Add the dictionary we just created.
            
            
            [children addObject:newLevelLookup];
            
            [currentLevelLookup setValue:children forKey:elementName];
            
        }
    }
    else
    {
        // This is a new element, so store the dictionary we just created.
        [currentLevelLookup setValue:newLevelLookup forKey:elementName];
        
        
        if([elementName isEqual:@"Disk"])
        {
            
            [newLevelLookup setValue:idVal forKey:@"resource-id"];
            [newLevelLookup setValue:model forKey:@"Model"];
            [newLevelLookup setValue:serialNum forKey:@"Serial"];
            [newLevelLookup setValue:firmVersion forKey:@"FirmwareVersion"];
            [newLevelLookup setValue:RPM forKey:@"RPM"];
            [newLevelLookup setValue:sectors forKey:@"Sectors"];
            [newLevelLookup setValue:capacity forKey:@"Capacity"];
            [newLevelLookup setValue:temperature forKey:@"Temperature"];
            [newLevelLookup setValue:ADAError forKey:@"ATAError"];
            [newLevelLookup setValue:slotName forKey:@"SlotName"];
            [newLevelLookup setValue:hardwareInterface forKey:@"HardwareInterface"];
            [newLevelLookup setValue:volName forKey:@"PoolName"];
            [newLevelLookup setValue:volState forKey:@"PoolState"];
            [newLevelLookup setValue:volHostID forKey:@"PoolHostId"];
            [newLevelLookup setValue:diskState forKey:@"DiskState"];
            [newLevelLookup setValue:channel forKey:@"Channel"];
            
        }
        if([elementName isEqual:@"Volume"])
        {
            [newLevelLookup setValue:isDeviceConnected forKey:@"ejectable"];
        }
        
        if([elementName isEqual:@"volume"])
        {
            [newLevelLookup setValue:free forKey:@"free"];
            [newLevelLookup setValue:size forKey:@"size"];
            [newLevelLookup setValue:USBName forKey:@"name"];
        }
        
        if ([elementName isEqual:@"Log"])
        {
            [newLevelLookup setValue:eventId forKey:@"eventId"];
            [newLevelLookup setValue:timestamp forKey:@"timestamp"];
            [newLevelLookup setValue:localized_ts forKey:@"localized_ts"];
            [newLevelLookup setValue:severity forKey:@"severity"];
            [newLevelLookup setValue:category forKey:@"category"];
            [newLevelLookup setValue:message_tag forKey:@"message-tag"];
        }
        if ([elementName isEqual:@"Share"])
        {
            [newLevelLookup setValue:share_TypeShare forKey:@"resource-type"];
        }
        if ([elementName isEqual:@"RAID"])
        {
            [newLevelLookup setValue:RAIDLevel forKey:@"Level"];
        }
        if([elementName isEqual:@"FirmwareProgress"])
        {
            
            [newLevelLookup setValue:value forKey:@"value"];
            [newLevelLookup setValue:state forKey:@"state"];
            
        }
        if([elementName isEqual:@"RemoteAccessSSH"])
        {
            
            [newLevelLookup setValue:secureStatus forKey:@"enable"];
            [newLevelLookup setValue:portNumber forKey:@"port"];
            
        }
        
        if([elementName isEqual:@"AlertConfig"])
        {
            [newLevelLookup setValue:alertContact forKey:@"contact"];
            [newLevelLookup setValue:user forKey:@"user"];
            [newLevelLookup setValue:password forKey:@"pass"];
            [newLevelLookup setValue:from forKey:@"from"];
            [newLevelLookup setValue:server forKey:@"server"];
            [newLevelLookup setValue:port forKey:@"port"];
            [newLevelLookup setValue:tls forKey:@"tls"];
        }
        
        if([elementName isEqual:@"Monitor"])
        {
            [newLevelLookup setValue:volume_ac forKey:@"volume"];
            [newLevelLookup setValue:disk_failure forKey:@"disk_failure"];
            [newLevelLookup setValue:fan forKey:@"fan"];
            [newLevelLookup setValue:disk_full forKey:@"disk_full"];
            [newLevelLookup setValue:board_temp forKey:@"board_temp"];
            [newLevelLookup setValue:psu forKey:@"psu"];
            [newLevelLookup setValue:ups forKey:@"ups"];
            [newLevelLookup setValue:power forKey:@"power"];
            [newLevelLookup setValue:disk_temp forKey:@"disk_temp"];
            [newLevelLookup setValue:quota_exceed forKey:@"quota_exceed"];
            
        }
        
        if([elementName isEqual:@"xs:response"])
        {
            
            [newLevelLookup setValue:name forKey:@"status"];
            
        }
        
        if([elementName isEqual:@"Log_Collection"])
        {
            
            [newLevelLookup setValue:log_entries forKey:@"n_log"];
            [newLevelLookup setValue:currentLogEntries forKey:@"n_current_logs"];
        }
        
        if([elementName isEqual:@"xs:get-s"])
        {
            
            [newLevelLookup setValue:resourceType forKey:@"resourceType"];
            
        }
        if([elementName isEqual:@"Protocol"])
        {
            [newLevelLookup setValue:nameTag forKey:@"id"];
            [newLevelLookup setValue:enabledStatus forKey:@"enabled"];
            
        }
        
        if([elementName isEqual:@"Fan"])
        {
            
            [newLevelLookup setValue:min_fan forKey:@"min_fan"];
            [newLevelLookup setValue:max_fan forKey:@"max_fan"];
            
        }
        
        if([elementName isEqual:@"args"])
        {
            
            [newLevelLookup setValue:args forKey:@"args"];
            
        }
        
        
        // Remember that this object should not stay on the queue
        _needRemoval = YES;
    }
    
    // Push the new dictionary on the queue
    [_currentLevel addObject:newLevelLookup];
    
    
}

-(void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    string = [string stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"\n "]];
    
    if(string != nil && string.length > 0)
    {
        if(_needRemoval)
            [_currentLevel removeLastObject];
        
        // Store the value in the current level.
        NSMutableDictionary* level = [_currentLevel objectAtIndex:_currentLevel.count-1];
        level[_lastElementName] = string;
        
        _needRemoval = NO;
    }
}

-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{
    // Element ended, so make sure to pop it off if we haven't done that already.
    if(_needRemoval)
        [_currentLevel removeLastObject];
    
    _needRemoval = YES;
}

@end